<?php
namespace Pinpoint\Conesso\Model\Api\Management\Process;

use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Framework\Api\SearchCriteria;
use Pinpoint\Conesso\Api\Data\CartSyncInterface;
use Pinpoint\Conesso\Api\Repository\CartsSyncRepositoryInterface;
use Pinpoint\Conesso\Api\Transfer\RequestInterface;
use Pinpoint\Conesso\Model\Api\Management\Log\Carts\LogCartInterface;
use Pinpoint\Conesso\Model\Api\Management\Transfer\Request\CartRequest;
use Pinpoint\Conesso\Model\Config\CartSettings;
use Pinpoint\Conesso\Model\ResourceModel\SyncCarts\CollectionFactory;

class Cart extends AbstractProcess
{
    /**
     * @var CartsSyncRepositoryInterface
     */
    private CartsSyncRepositoryInterface $cartsSyncRepository;

    /**
     * @var CollectionFactory
     */
    private CollectionFactory $cartsCollectionFactory;

    /**
     * @var LogCartInterface
     */
    private LogCartInterface $logCart;

    /**
     * @var CartSettings
     */
    private CartSettings $cartSettings;

    /**
     * @var CartRepositoryInterface
     */
    private CartRepositoryInterface $cartRepository;

    /**
     * @param CartsSyncRepositoryInterface $cartsSyncRepository
     * @param CartRepositoryInterface $cartRepository
     * @param CollectionFactory $cartsCollectionFactory
     * @param LogCartInterface $logCart
     * @param CartSettings $cartSettings
     * @param CreateContext $context
     */
    public function __construct(
        CartsSyncRepositoryInterface $cartsSyncRepository,
        CartRepositoryInterface      $cartRepository,
        CollectionFactory            $cartsCollectionFactory,
        LogCartInterface             $logCart,
        CartSettings                 $cartSettings,
        CreateContext                $context
    )
    {
        $this->cartsSyncRepository = $cartsSyncRepository;
        $this->cartRepository = $cartRepository;
        $this->cartsCollectionFactory = $cartsCollectionFactory;
        $this->logCart = $logCart;
        $this->cartSettings = $cartSettings;
        parent::__construct($context);
    }

    /**
     * @param array $params
     * @return bool
     */
    public function synchronizeData(array $params = []) : bool
    {
        $batchSize = $this->cartSettings->getBatchSize() ?? 75;
        $carts = $this->cartsSyncRepository->getList($this->getSearchCriteria($batchSize));
        $this->getRequestProvider()->sendRequest(['items' => $carts]);
        return true;
    }

    /**
     * @return RequestInterface
     */
    private function getRequestProvider(): RequestInterface
    {
        return $this->requestProvider->getRequestProvider(CartRequest::class);
    }

    /**
     * @param int $batchSize
     * @return SearchCriteria
     */
    protected function getSearchCriteria(int $batchSize = 75): SearchCriteria
    {
        return $this->searchCriteriaBuilder
            ->addFilter(CartSyncInterface::IS_SYNCED, 0)
            ->create()
            ->setPageSize($batchSize);
    }

    /**
     * @return void
     */
    public function migrateData(): void
    {
        try {
            $this->truncateSyncTable();
            $carts = $this->getAllCartsInMagento();

            foreach($carts as $cart) {
                $this->logCart->logCart($cart, CartSyncInterface::EVENT_TYPE_SAVE);
            }
            $this->synchronizeData();
            $this->truncateSyncTable();
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @return array
     */
    private function getAllCartsInMagento(): array
    {
        if($this->cartSettings->getSyncFromDate()) {
            $criteria = $this->searchCriteriaBuilder
                ->addFilter(CartInterface::KEY_CREATED_AT, $this->cartSettings->getSyncFromDate(), 'gteq')
                ->create();
        } else {
            $criteria = $this->searchCriteriaBuilder->create();
        }

        return $this->cartRepository->getList($criteria)->getItems();
    }

    /**
     * Truncate "connesso_cart_sync"
     * @return void
     */
    private function truncateSyncTable(): void
    {
        $collection = $this->cartsCollectionFactory->create();
        $connection = $collection->getConnection();
        $tableName = $collection->getMainTable();
        $connection->truncateTable($tableName);
    }
}
