<?php
namespace Pinpoint\Conesso\Model\Api\Management\Process;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Framework\Api\SearchCriteria;
use Magento\Framework\Exception\LocalizedException;
use Pinpoint\Conesso\Api\Data\ContactSyncInterface;
use Pinpoint\Conesso\Api\Repository\ContactsRepositoryInterface;
use Pinpoint\Conesso\Api\Transfer\RequestInterface;
use Pinpoint\Conesso\Model\Api\Management\Log\Customer\LogCustomerInterface;
use Pinpoint\Conesso\Model\Api\Management\Transfer\Request\CustomerRequest;
use Pinpoint\Conesso\Model\Config\ContactSettings;
use Pinpoint\Conesso\Model\ResourceModel\SyncContacts\CollectionFactory;

class Customer extends AbstractProcess
{
    /**
     * @var ContactsRepositoryInterface
     */
    private ContactsRepositoryInterface $contactRepository;

    /**
     * @var CollectionFactory
     */
    private CollectionFactory $contactsCollectionFactory;

    /**
     * @var LogCustomerInterface
     */
    private LogCustomerInterface $logCustomer;

    /**
     * @var ContactSettings
     */
    private ContactSettings $contactSettings;

    /**
     * @var CustomerRepositoryInterface
     */
    private CustomerRepositoryInterface $customerRepository;

    /**
     * @param ContactsRepositoryInterface $contactRepository
     * @param CustomerRepositoryInterface $customerRepository
     * @param CollectionFactory $contactsCollectionFactory
     * @param LogCustomerInterface $logCustomer
     * @param ContactSettings $contactSettings
     * @param CreateContext $context
     */
    public function __construct(
        ContactsRepositoryInterface $contactRepository,
        CustomerRepositoryInterface $customerRepository,
        CollectionFactory           $contactsCollectionFactory,
        LogCustomerInterface        $logCustomer,
        ContactSettings             $contactSettings,
        CreateContext               $context
    ) {
        $this->contactRepository = $contactRepository;
        $this->customerRepository = $customerRepository;
        $this->contactsCollectionFactory = $contactsCollectionFactory;
        $this->logCustomer = $logCustomer;
        $this->contactSettings = $contactSettings;
        parent::__construct($context);
    }

    /**
     * @param array $params
     * @return bool
     */
    public function synchronizeData(array $params = []) : bool
    {
        $batchSize = $this->contactSettings->getBatchSize() ?? 75;
        $customers = $this->contactRepository->getList($this->getSearchCriteria($batchSize));
        $this->getRequestProvider()->sendRequest(['items' => $customers]);
        return true;
    }

    /**
     * @return void
     */
    public function migrateData(): void
    {
        try {
            $this->truncateSyncTable();
            $customers = $this->getAllCustomersInMagento();

            foreach($customers as $customer) {
                $this->logCustomer->logCustomer($customer, ContactSyncInterface::EVENT_TYPE_SAVE);
            }
            $this->synchronizeData();
            $this->truncateSyncTable();
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @return RequestInterface
     */
    private function getRequestProvider(): RequestInterface
    {
        return $this->requestProvider->getRequestProvider(CustomerRequest::class);
    }

    /**
     * @param int $batchSize
     * @return SearchCriteria
     */
    protected function getSearchCriteria(int $batchSize = 75): SearchCriteria
    {
        return $this->searchCriteriaBuilder
            ->addFilter(ContactSyncInterface::IS_SYNCED, '0', 'eq')
            ->create()
            ->setPageSize($batchSize);
    }

    /**
     * @return CustomerInterface[]
     * @throws LocalizedException
     */
    private function getAllCustomersInMagento(): array
    {
        if($this->contactSettings->getSyncFromDate()) {
            $criteria = $this->searchCriteriaBuilder
                ->addFilter(ProductInterface::CREATED_AT, $this->contactSettings->getSyncFromDate(), 'gteq')
                ->create();
        } else {
            $criteria = $this->searchCriteriaBuilder->create();
        }

        return $this->customerRepository->getList($criteria)->getItems();
    }

    /**
     * @return void
     */
    private function truncateSyncTable(): void
    {
        $collection = $this->contactsCollectionFactory->create();
        $connection = $collection->getConnection();
        $tableName = $collection->getMainTable();
        $connection->truncateTable($tableName);
    }
}
