<?php
namespace Pinpoint\Conesso\Model\Api\Management\Process;

use Exception;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\Api\SearchCriteria;
use Pinpoint\Conesso\Api\Data\OrderSyncInterface;
use Pinpoint\Conesso\Api\Repository\OrdersSyncRepositoryInterface;
use Pinpoint\Conesso\Api\Transfer\RequestInterface;
use Pinpoint\Conesso\Model\Api\Management\Log\Orders\LogOrderInterface;
use Pinpoint\Conesso\Model\Api\Management\Transfer\Request\OrderRequest;
use Pinpoint\Conesso\Model\Config\OrderSettings;
use Pinpoint\Conesso\Model\ResourceModel\SyncOrders\CollectionFactory;

class Order extends AbstractProcess
{
    /**
     * @var OrdersSyncRepositoryInterface
     */
    private OrdersSyncRepositoryInterface $ordersSyncRepository;

    /**
     * @var CollectionFactory
     */
    private CollectionFactory $ordersCollectionFactory;

    /**
     * @var LogOrderInterface
     */
    private LogOrderInterface $logOrder;

    /**
     * @var OrderSettings
     */
    private OrderSettings $orderSettings;

    /**
     * @var OrderRepositoryInterface
     */
    private OrderRepositoryInterface $orderRepository;

    /**
     * @param OrdersSyncRepositoryInterface $ordersSyncRepository
     * @param OrderRepositoryInterface $orderRepository
     * @param CollectionFactory $ordersCollectionFactory
     * @param LogOrderInterface $logOrder
     * @param OrderSettings $orderSettings
     * @param CreateContext $context
     */
    public function __construct(
        OrdersSyncRepositoryInterface $ordersSyncRepository,
        OrderRepositoryInterface $orderRepository,
        CollectionFactory           $ordersCollectionFactory,
        LogOrderInterface           $logOrder,
        OrderSettings               $orderSettings,
        CreateContext               $context
    ) {
        $this->ordersSyncRepository = $ordersSyncRepository;
        $this->orderRepository = $orderRepository;
        $this->ordersCollectionFactory = $ordersCollectionFactory;
        $this->logOrder = $logOrder;
        $this->orderSettings = $orderSettings;
        parent::__construct($context);
    }

    /**
     * @param array $params
     * @return bool
     */
    public function synchronizeData(array $params = []) : bool
    {
        $batchSize = $this->orderSettings->getBatchSize() ?? 75;
        $orders = $this->ordersSyncRepository->getList($this->getSearchCriteria($batchSize));
        $this->getRequestProvider()->sendRequest(['items' => $orders]);
        return true;
    }

    /**
     * @return void
     */
    public function migrateData(): void
    {
        try {
            $this->truncateSyncTable();
            $orders = $this->getAllOrdersInMagento();

            foreach($orders as $order) {
                $this->logOrder->logOrder($order, OrderSyncInterface::EVENT_TYPE_SAVE);
            }
            $this->synchronizeData();
            $this->truncateSyncTable();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @return RequestInterface
     */
    private function getRequestProvider(): RequestInterface
    {
        return $this->requestProvider->getRequestProvider(OrderRequest::class);
    }

    /**
     * @param int $batchSize
     * @return SearchCriteria
     */
    protected function getSearchCriteria(int $batchSize = 75): SearchCriteria
    {
        return $this->searchCriteriaBuilder
            ->addFilter(OrderSyncInterface::IS_SYNCED, '0')
            ->create()
            ->setPageSize($batchSize);
    }

    /**
     * @return array
     */
    private function getAllOrdersInMagento(): array
    {
        if($this->orderSettings->getSyncFromDate()) {
            $criteria = $this->searchCriteriaBuilder
                ->addFilter(OrderInterface::CREATED_AT, $this->orderSettings->getSyncFromDate(), 'gteq')
                ->create();
        } else {
            $criteria = $this->searchCriteriaBuilder->create();
        }

        return $this->orderRepository->getList($criteria)->getItems();
    }

    /**
     * Truncate "connesso_order_sync"
     * @return void
     */
    private function truncateSyncTable(): void
    {
        $collection = $this->ordersCollectionFactory->create();
        $connection = $collection->getConnection();
        $tableName = $collection->getMainTable();
        $connection->truncateTable($tableName);
    }
}
