<?php
namespace Pinpoint\Conesso\Model\Api\Management\Transfer\Request;

use Exception;
use Magento\Sales\Model\Order\Item;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;
use Magento\Framework\DataObject;
use Pinpoint\Conesso\Api\Data\OrderSyncInterface;
use Pinpoint\Conesso\Api\Repository\OrdersSyncRepositoryInterface;
use Pinpoint\Conesso\Model\Api\Access\Auth;
use Pinpoint\Conesso\Model\Config\OrderSettings;

class OrderRequest extends AbstractRequest
{
    /**
     * @var OrdersSyncRepositoryInterface
     */
    protected OrdersSyncRepositoryInterface $ordersSyncRepository;

    /**
     * @var CollectionFactory
     */
    protected CollectionFactory $orderCollectionFactory;

    /**
     * @var OrderSettings
     */
    private OrderSettings $orderSettings;

    /**
     * @var CartRequest
     */
    private CartRequest $cartRequest;

    /**
     * @param Auth $authorization
     * @param OrdersSyncRepositoryInterface $ordersSyncRepository
     * @param CollectionFactory $orderCollectionFactory
     * @param OrderSettings $orderSettings
     * @param CartRequest $cartRequest
     */
    public function __construct(
        Auth $authorization,
        OrdersSyncRepositoryInterface $ordersSyncRepository,
        CollectionFactory $orderCollectionFactory,
        OrderSettings $orderSettings,
        CartRequest $cartRequest
    ) {
        $this->ordersSyncRepository = $ordersSyncRepository;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->orderSettings = $orderSettings;
        $this->cartRequest = $cartRequest;
        parent::__construct($authorization);
    }

    /**
     * @param mixed $order
     * @return array
     */
    public function buildData(mixed $order): array
    {
        $data = [
            'contactId' => (!$order->getCustomerIsGuest()) ? $this->cartRequest->getContactIdByEmail($order->getCustomerEmail()) : '',
            'customerEmail' => $order->getCustomerEmail(),
            'customerFirstname' => $order->getCustomerFirstname(),
            'customerLastname' => $order->getCustomerLastname(),
            'customerGender' => $this->getCustomerGenderLabel($order->getCustomerGender()),
            'customerTelephone' => $order->getShippingAddress()->getTelephone(),
            'customerMobile' => $order->getShippingAddress()->getTelephone(),
            'customerDob' => null,
            'customerIsGuest' => $order->getCustomerIsGuest(),
            'customerNotes' => $order->getCustomerNote(),
            'billingFirstname' => $order->getBillingAddress()->getFirstname(),
            'billingLastname' => $order->getBillingAddress()->getLastname(),
            'billingAddress1' => $order->getBillingAddress()->getStreet()[0],
            'billingAddress2' => isset($order->getBillingAddress()->getStreet()[1]) ? $order->getBillingAddress()->getStreet()[1] : '',
            'billingCity' => $order->getBillingAddress()->getCity(),
            'billingCounty' => $order->getBillingAddress()->getRegion(),
            'billingPostcode' => $order->getBillingAddress()->getPostcode(),
            'billingCountry' => $order->getBillingAddress()->getCountryId(),
            'shippingFirstname' => $order->getShippingAddress()->getFirstname(),
            'shippingLastname' => $order->getShippingAddress()->getLastname(),
            'shippingAddress1' => $order->getShippingAddress()->getStreet()[0],
            'shippingAddress2' => isset($order->getShippingAddress()->getStreet()[1]) ? $order->getShippingAddress()->getStreet()[1] : '',
            'shippingCity' => $order->getShippingAddress()->getCity(),
            'shippingCounty' => $order->getShippingAddress()->getRegion(),
            'shippingCountry' => $order->getShippingAddress()->getCountryId(),
            'shippingPostcode' => $order->getShippingAddress()->getPostcode(),
            'total' => $order->getGrandTotal() - $order->getTaxAmount(),
            'totalIncTax' => $order->getGrandTotal(),
            'isoCurrencyCode' => $order->getOrderCurrencyCode(),
            'numberOfProducts' => $order->getTotalItemCount(),
            'cartProducts' => $this->getOrderProducts($order)
        ];

        return array_merge($data, $this->getCustomAttributes($order));
    }

    /**
     * @param $order
     * @return array|array[]
     */
    private function getOrderProducts($order): array
    {
        return array_map(function (Item $item) {
            return [
                'productId' => 'api-' . $item->getSku(),
                'productSku' => $item->getSku(),
                'productName' => $item->getName(),
                'productDescription' => $item->getDescription(),
                'taxPercent' => $item->getTaxPercent(),
                'productPriceTax' => $item->getTaxAmount(),
                'productOriginalPrice' => $item->getOriginalPrice(),
                'productPrice' => $item->getPrice(),
                'productPriceIncTax' => $item->getPriceInclTax(),
                'rowTotal' => $item->getRowTotal(),
                'rowTotalIncTax' =>  $item->getRowTotalInclTax(),
                'quantity' => $item->getQtyOrdered()
            ];
        }, $order->getAllVisibleItems());
    }

    /**
     * @param $order
     * @return array
     */
    private function getCustomAttributes($order): array
    {
        $data = [];
        $customAttributes = $this->orderSettings->getCustomAttributes();
        if($customAttributes) {
            foreach ($customAttributes as $key => $row) {
                $conessoAttribute = $row['conesso_attribute'];
                $magentoAttribute = $row['magento_attribute'];
                if($conessoAttribute && $magentoAttribute) {
                    $data[$conessoAttribute] = $order->getData($magentoAttribute);
                }
            }
        }

        return $data;
    }

    /**
     * @param mixed $data
     * @return bool
     */
    public function sendRequest(mixed $data): bool
    {
        //TODO: for order
        foreach ($data['items'] as $item) {
            if ($item->getEventType() == OrderSyncInterface::EVENT_TYPE_SAVE) {
                $this->saveOrder($item);
            }
            if ($item->getEventType() == OrderSyncInterface::EVENT_TYPE_CANCELLED)
            {
                $this->cancelOrderInConesso($item);
            }
            $this->updatedSynchronizedData($item);
        }
        return true;
    }

    /**
     * @param $item
     * @return void
     */
    private function updatedSynchronizedData($item): void
    {
        $item->setIsSynced(1);
        $this->ordersSyncRepository->save($item);
    }

    /**
     * @param $item
     * @return void
     */
    private function saveOrder($item): void
    {
        //TODO: Load Order
        if ($order = $this->getOrderById($item->getOrderId())) {
            $parameters = $this->buildData($order);
            $this->getConnessoClient()->orders()->create($parameters);
//            $customerEmail = $order->getEmail();
//            if(!$this->existsOrderInConesso($customerEmail)) {
//                $this->getConnessoClient()->orders()->create($parameters);
//            }
        }
    }

    /**
     * @param $customerEmail
     * @return false|mixed
     */
    private function existsOrderInConesso($customerEmail): mixed
    {
        try {
            $contacts = $this->getConnessoClient()->contacts()->list()->data;

            foreach ($contacts as $contact) {
                if ($contact['email'] == $customerEmail) {
                    return $contact['id'];
                }
            }
            return false;
        } catch (TypeError|Exception $e) {
            return false;
        }
    }

    /**
     * @param $customerId
     * @return false|DataObject
     */
    private function getOrderById($customerId): DataObject|bool
    {
        try {
            return $this->orderCollectionFactory->create()->addFieldToFilter('entity_id', $customerId)->getFirstItem();
        } catch (Exception $e) {
            return  false;
        }
    }

    /**
     * @param $item
     * @return void
     */
    private function cancelOrderInConesso($item): void
    {
        $orderId = $this->existsOrderInConesso($item->getEmail());
        $this->getConnessoClient()->orders()->delete($orderId);
    }

    /**
     * @param $genderId
     * @return string
     */
    private function getCustomerGenderLabel($genderId): string
    {
        return match ($genderId) {
            '1' => 'Male',
            '2' => 'Female',
            default => 'Not Specified',
        };
    }
}
