<?php
namespace Pinpoint\Conesso\Model\Api\Management\Transfer\Request;

use Exception;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Helper\Data as TaxHelper;
use Magento\CatalogInventory\Model\Stock\StockItemRepository;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Framework\Exception\NoSuchEntityException;
use Pinpoint\Conesso\Api\Data\ProductSyncInterface;
use Pinpoint\Conesso\Model\Api\Access\Auth;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Pinpoint\Conesso\Api\Repository\ProductsSyncRepositoryInterface;
use Pinpoint\Conesso\Model\Config\ProductSettings;
use TypeError;

class ProductRequest extends AbstractRequest
{
    /**
     * @var TaxHelper
     */
    private TaxHelper $taxHelper;

    /**
     * @var StockItemRepository
     */
    private StockItemRepository $stockItemRepository;

    /**
     * @var ProductRepositoryInterface
     */
    private ProductRepositoryInterface $productRepository;

    /**
     * @var ProductsSyncRepositoryInterface
     */
    private ProductsSyncRepositoryInterface $productsSyncRepository;

    /**
     * @var ProductSettings
     */
    private ProductSettings $productSettings;

    /**
     * @var Configurable
     */
    private Configurable $configurable;

    /**
     * @param TaxHelper $taxHelper
     * @param StockItemRepository $stockItemRepository
     * @param Auth $authorization
     * @param ProductRepositoryInterface $productRepository
     * @param ProductsSyncRepositoryInterface $productsSyncRepository
     * @param ProductSettings $productSettings
     * @param Configurable $configurable
     */
    public function __construct(
        TaxHelper $taxHelper,
        StockItemRepository $stockItemRepository,
        Auth $authorization,
        ProductRepositoryInterface $productRepository,
        ProductsSyncRepositoryInterface $productsSyncRepository,
        ProductSettings $productSettings,
        Configurable $configurable
    ) {
        $this->taxHelper = $taxHelper;
        $this->stockItemRepository = $stockItemRepository;
        $this->productRepository = $productRepository;
        $this->productsSyncRepository = $productsSyncRepository;
        $this->productSettings = $productSettings;
        $this->configurable = $configurable;
        parent::__construct($authorization);
    }

    /**
     * @param mixed $product
     * @return array
     * @throws NoSuchEntityException
     */
    public function buildData(mixed $product): array
    {
        $data = [
            'id' => $product->getId(),
            'apiReferenceId' => 'api-' . $product->getSku(),
            'name' => $product->getName(),
            'description' => $product->getDescription() ?? null,
            'barcode' => $product->getUrlKey(),
            'uri' => $product->getProductUrl(),
            'sku' => $product->getSku(),
            'dateCreated' => date('Y-m-d\TH:i:s\Z',strtotime($product->getCreatedAt())),
            'dateUpdated' => date('Y-m-d\TH:i:s\Z',strtotime($product->getUpdatedAt())),
            'groupType' => $this->getProductGroupType($product),
            'priceTax' => $this->getTaxPrice($product),
            'priceIncTax' => $this->getPriceInclTax($product),
            'originalPrice' => $this->getPriceWithoutTax($product),
            'price' => $this->getPriceWithoutTax($product),
            'weight' => (int) $product->getWeight(),
            'weightUnit' => $this->productSettings->getWeightUnit() == 'lbs' ? 'lb' : 'kg',
            'stockQuantity' => $this->getStock($product),
            'visibility' => $product->getVisibility() == 1 ? 'not-individually' : 'public',
            'status' => $product->getStatus() == '1' ? 'active' : 'disabled'
        ];

        return array_merge($data, $this->getCustomAttributes($product));
    }

    /**
     * @param $product
     * @return array
     */
    private function getCustomAttributes($product): array
    {
        $data = [];
        $customAttributes = $this->productSettings->getCustomAttributes();
        if($customAttributes) {
            foreach ($customAttributes as $key => $row) {
                $conessoAttribute = $row['conesso_attribute'];
                $magentoAttribute = $row['magento_attribute'];
                if($conessoAttribute && $magentoAttribute) {
                    $data[$conessoAttribute] = $product->getAttributeText($magentoAttribute);
                }
            }
        }

        return $data;
    }

    /**
     * @param mixed $data
     * @return bool
     * @throws NoSuchEntityException
     */
    public function sendRequest(mixed $data) : bool
    {
        foreach ($data['items'] as $item) {
            if ($item->getEventType() == ProductSyncInterface::EVENT_TYPE_SAVE) {
                $this->saveProduct($item);
            }
            if ($item->getEventType() == ProductSyncInterface::EVENT_TYPE_DELETE)
            {
                $this->deleteProductInConesso($item);
            }
            $this->updatedSynchronizedData($item);
        }
        return false;
    }

    /**
     * @param $item
     * @return void
     */
    private function updatedSynchronizedData($item): void
    {
        $item->setIsSynced(1);
        $this->productsSyncRepository->save($item);
    }

    /**
     * @param $item
     * @return void
     * @throws NoSuchEntityException
     */
    private function saveProduct($item): void
    {
        $sku = $item->getProductSku();
        if($product = $this->getProductBySku($sku)) {
            $parameters = $this->buildData($product);
            $sku = $product->getSku();
            if($this->existsProductConesso($sku)) {
                $this->updateProductInConesso($sku, $parameters);
            } else {
                $this->createProductInConesso($parameters);
            }
        }
    }

    /**
     * @param string $sku
     * @return bool
     */
    private function existsProductConesso(string $sku): bool
    {
        try {
            $this->getConnessoClient()->products()->retrieve($sku);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @param $parameters
     * @return void
     */
    private function createProductInConesso($parameters): void
    {
        $this->getConnessoClient()->products()->create($parameters);
    }

    /**
     * @param $sku
     * @param $parameters
     * @return void
     */
    private function updateProductInConesso($sku, $parameters): void
    {
        $this->getConnessoClient()->products()->update($sku, $parameters);
    }


    /**
     * @param $item
     * @return void
     */
    private function deleteProductInConesso($item): void
    {
        $this->getConnessoClient()->products()->delete($item->getProductSku());
    }

    /**
     * @param $product
     * @return float
     * @throws NoSuchEntityException
     */
    private function getStock($product): float
    {
        return $this->stockItemRepository->get($product->getId())->getQty();
    }

    /**
     * @param $sku
     * @return ProductInterface|null
     */
    private function getProductBySku($sku): ?ProductInterface
    {
        try {
            return $this->productRepository->get($sku);
        } catch (Exception $e) {
            return null;
        }
    }

    /**
     * @param $product
     * @return float
     */
    private function getTaxPrice($product): float
    {
        return $this->taxHelper->getTaxPrice($product, $product->getFinalPrice(), true);
    }

    /**
     * @param $product
     * @return float
     */
    private function getPriceWithoutTax($product): float
    {
        return round($product->getPriceInfo()->getPrice('final_price')->getAmount()->getBaseAmount(), 2);
    }

    /**
     * @param $product
     * @return float
     */
    private function getPriceInclTax($product): float
    {
        return round($product->getPriceInfo()->getPrice('final_price')->getAmount()->getValue(),2);
    }

    /**
     * @param $product
     * @return string
     */
    private function getProductGroupType($product): string
    {
        if(empty($this->configurable->getParentIdsByChild($product->getId()))) {
            return 'standalone-product';
        }

        return 'child-product';
    }
}
