<?php

declare(strict_types=1);

namespace Pinpoint\Conesso\Model\Api;

use GuzzleHttp\Client;
use GuzzleHttp\ClientFactory;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\ResponseFactory;
use Magento\Framework\Webapi\Rest\Request;
use Pinpoint\Conesso\Api\EndpointsInterface;
use Pinpoint\Conesso\Helper\Data as ConessoDataHelper;

class Service
{
    /*
     * @var ResponseFactory
     */
    private ResponseFactory $responseFactory;

    /**
     * @var ClientFactory
     */
    private ClientFactory $clientFactory;

    /**
     * @var ConessoDataHelper
     */
    protected ConessoDataHelper $conessoDataHelper;

    /**
     * GitApiService constructor
     *
     * @param ClientFactory $clientFactory
     * @param ResponseFactory $responseFactory
     * @param ConessoDataHelper $conessoDataHelper
     */
    public function __construct(
        ClientFactory   $clientFactory,
        ResponseFactory $responseFactory,
        ConessoDataHelper $conessoDataHelper
    ) {
        $this->clientFactory = $clientFactory;
        $this->responseFactory = $responseFactory;
        $this->conessoDataHelper = $conessoDataHelper;
    }

    /**
     * @param string|null $mode
     * @return array
     */
    public function checkConnection(?string $mode)
    {
        if (!$mode) {
            $mode = $this->conessoDataHelper->getApiMode();
        }

        $url = EndpointsInterface::CONTACTS['endpoint'];
        $response = $this->doRequest($url, $mode);
        $responseBody = $response->getBody();
        $responseContent = $responseBody->getContents();// API response in JSON format
        $result['message'] = '';
        if ($responseContent) {
            try {
                $responseContentDecoded = $this->conessoDataHelper->serializer->unserialize($responseContent);
                if ($responseContentDecoded && isset($responseContentDecoded['metaData'])) {
                    $result['message'] = "Conesso: {$mode} connection successful!";
                }
            } catch (\InvalidArgumentException $e) {
                //do nothing
            }
        }

        return $result;
    }

    /**
     * @param array $params
     * @param string $method
     * @param string|null $mode
     * @return Response
     */
    public function carts(array $params, string $method, ?string $mode = null): Response
    {
        if (!$mode) {
            $mode = $this->conessoDataHelper->getApiMode();
        }

        return $this->doRequest(
            EndpointsInterface::CARTS['endpoint'],
            $mode,
            $params,
            $method
        );
    }

    public function products(array $params, string $method, ?string $productSku = null, ?string $mode = null): Response
    {
        if (!$mode) {
            $mode = $this->conessoDataHelper->getApiMode();
        }

        $endpoint = EndpointsInterface::PRODUCTS['endpoint'];
        if ($productSku) {
            $endpoint = str_replace('{id}', $productSku, EndpointsInterface::PRODUCTS_ID['endpoint']);
        }

        return $this->doRequest(
            $endpoint,
            $mode,
            $params,
            $method
        );
    }

    /**
     * @param string $mode
     * @return string|null
     */
    private function getApiKey(string $mode): ?string
    {
        return $mode == 'prod' ? $this->conessoDataHelper->getApiKeyProd() : $this->conessoDataHelper->getApiKeyTest();
    }

    /**
     * @param string $mode
     * @return string
     */
    private function getApiUrl(string $mode): string
    {
        return $mode == 'prod' ? EndpointsInterface::CONESSO_PRODUCTION_URL : EndpointsInterface::CONESSO_QA_URL;
    }

    /**
     * Do API request with provided params
     *
     * @param string $uriEndpoint
     * @param string $mode
     * @param array $params
     * @param string $requestMethod
     *
     * @return Response
     */
    private function doRequest(
        string $uriEndpoint,
        string $mode,
        array  $params = [],
        string $requestMethod = Request::HTTP_METHOD_GET
    ): Response
    {
        $uri = $this->getApiUrl($mode);

        /** @var Client $client */
        $client = $this->clientFactory->create(
            [
                'config' => [
                    'base_uri' => $uri,
                ]
            ]
        );

        $params['headers']['api_key'] = $this->getApiKey($mode);

        try {
            $response = $client->request(
                $requestMethod,
                sprintf($uriEndpoint, $uri),
                $params
            );
        } catch (GuzzleException $exception) {
            /** @var Response $response */
            $response = $this->responseFactory->create([
                'status' => $exception->getCode(),
                'reason' => $exception->getMessage(),
                'body' => $exception->getResponse()->getBody()
            ]);
        }

        return $response;
    }
}
